package org.gcube.portlets.widgets.ckancontentmoderator.client;

import java.util.List;

import org.gcube.datacatalogue.utillibrary.shared.ItemStatus;
import org.gcube.portlets.widgets.ckancontentmoderator.shared.CatalogueDataset;
import org.gcube.portlets.widgets.ckancontentmoderator.shared.SearchedData;

import com.google.gwt.user.client.rpc.RemoteService;
import com.google.gwt.user.client.rpc.RemoteServiceRelativePath;


/**
 * The client side stub for the RPC service.
 *
 * @author Francesco Mangiacrapa at ISTI-CNR francesco.mangiacrapa@isti.cnr.it
 * 
 * Jun 14, 2021
 */
@RemoteServiceRelativePath("greet")
public interface CkanContentModeratorService extends RemoteService {

	/**
	 * Checks if is content moderator enabled.
	 *
	 * @return true, if is content moderator enabled
	 */
	public boolean isContentModeratorEnabled();

	
	/**
	 * Sets the status.
	 *
	 * @param theStatus the the status
	 * @param itemNames the item names
	 */
	public void setStatus(ItemStatus theStatus, List<String> itemNames);


	/**
	 * Approve item.
	 *
	 * @param itemNames the item names
	 */
	public void approveItem(List<String> itemNames);


	/**
	 * Reject item.
	 *
	 * @param itemName the item name
	 * @param permanentlyDelete the permanently delete
	 * @param reasonMsg the reason msg
	 */
	public void rejectItem(String itemName, boolean permanentlyDelete, String reasonMsg);


	/**
	 * Permanently delete.
	 *
	 * @param itemNames the item names
	 */
	public void permanentlyDelete(List<String> itemNames);

	/**
	 * Gets the list items for status.
	 *
	 * @param theStatus the the status
	 * @param limit the limit
	 * @param offset the offset
	 * @return the list items for status
	 * @throws Exception the exception
	 */
	public List<CatalogueDataset> getListItemsForStatus(ItemStatus theStatus, int limit, int offset) throws Exception;

	/**
	 * Gets the data for status.
	 *
	 * @param status the status
	 * @param startIndex the start index
	 * @param lenght the lenght
	 * @param serverIndex the server index
	 * @return the data for status
	 * @throws Exception the exception
	 */
	SearchedData getDataForStatus(ItemStatus status, int startIndex, int lenght, int serverIndex) throws Exception;

}
